@@ -18,12 +18,21 @@
     unused_import_braces,
     unused_qualifications
 )]
+#![cfg_attr(not(feature = "std"), no_std)]
+
+#[cfg(feature = "std")]
+extern crate core;
+
+#[cfg(feature = "alloc")]
+extern crate alloc;
 
 pub(crate) mod atomic;
 
+mod portal;
 mod router;
 mod workers;
 
+pub(crate) use portal::*;
 pub(crate) use router::*;
 pub(crate) use workers::*;
 
@@ -33,7 +42,7 @@ mod transport;
 pub use error::*;
 pub use transport::*;
 
-use ockam_core::lib::net::SocketAddr;
+use ockam_core::compat::net::SocketAddr;
 use ockam_core::Result;
 
 /// TCP address type constant
